const radius = require('radius');
const dgram = require('dgram');
let redis = require('./redis')
redis.init()
const secret = 'secret'; // Replace with your shared secret
const port = 1812; // Default RADIUS authentication port
let reject = 'Access-Reject'
let accept = 'Access-Accept'
let maxConnection = 2

const server = dgram.createSocket('udp4');

server.on('message', (msg, rinfo) => {
    const packet = radius.decode({packet: msg, secret});

    if (packet.code !== 'Access-Request') {
        console.error('Invalid packet type: ' + packet.code);
        return;
    }

    const username = packet.attributes['User-Name'];
    if (username === 'root')
        return;
    const password = packet.attributes['User-Password'];
    let ip = packet.attributes['Calling-Station-Id']
    let user = {username, password, ip}

    // console.log('Received access request from:', username);


    doAuth(packet, user, rinfo)
});

server.on('listening', () => {
    const address = server.address();
    console.log('RADIUS server listening on port', address.port);
});

server.bind(port);

async function doAuth(packet, user, rinfo) {
    let existsUser = authenticateUser(user.username)
    if (!existsUser) {
        console.log('uuid with ' + user.username + ' with ip:' + user.ip + ' not exist')
        sendResponsePacket(packet, rinfo, reject)
        return
    }
    let keys = await redis.getAllKeysByUUID(user.username)
    if (keys.length >= 2) {
        console.log('uuid with ' + user.username + ' with ip:' + user.ip + ' reach limits')
        sendResponsePacket(packet, rinfo, reject)
        return
    }
    console.log('uuid with ' + user.username + ' with ip:' + user.ip + ' granted')
    sendResponsePacket(packet, rinfo, accept)
    redis.addIp(user.username, user.ip)

}

function authenticateUser(username) {
    //  console.log(username,password)
    // Replace this function with your actual authentication logic
    return username === 'ali';
}

function sendResponsePacket(packet, rinfo, code) {
    let response;
    response = radius.encode_response({
        packet,
        code: code,
        secret,
    });


    server.send(response, 0, response.length, rinfo.port, rinfo.address);
}
